@@ -9,13 +9,26 @@
                     model_type=NULL,
                     use_groups=NULL,
                     fixtype=NULL,...) {
+
   
   . <- NULL
   to_use <- stanmodels[['irt_standard_noid']]
+  
+  if(this_data$time_proc==4) {
+    tol_rel_obj <- .0001
+    eval_elbo <- 200
+  } else {
+    tol_rel_obj <- .0001
+    eval_elbo <- 100
+  }
 
   
   post_modes <- rstan::vb(object=to_use,data =this_data,
-                          algorithm='meanfield')
+                          algorithm='meanfield',
+                          refresh=this_data$id_refresh,
+                          eval_elbo=eval_elbo,
+                          tol_rel_obj=tol_rel_obj, # better convergence criterion than default
+                          output_samples=200)
 
   # Test whether there is a lot of missing data
   
@@ -85,6 +98,7 @@
     
     # what to constrain the difference to given the priors
     diff_high <- person[to_constrain_high[1]] 
+    sign_flip <- 1 # whether we need to flip signs 
     diff <- person[to_constrain_high[1]]  - person[to_constrain_low[1]]
     
   } else {
@@ -99,6 +113,11 @@
     
     
     diff_high <- abs(person[to_constrain_high])
+    if(sign(person[to_constrain_high])<0) {
+      sign_flip <- -1
+    } else {
+      sign_flip <- 1
+    }
     diff <- diff_high - sign(person[to_constrain_high])*person[to_constrain_low]
     
     # next we are going to re-order the person IDs around the constraints
@@ -123,25 +142,79 @@
   if(this_data$T>1) {
     # do some additional model identification if necessary for time-varying ideal pt models
     # figure out upper limit of estimated variances
+
+    ideal_pts_low <- rstan::extract(post_modes,'L_tp1')[[1]] %>% apply(c(2,3),quantile,.01) %>% .[,new_order]
+    ideal_pts_high <- rstan::extract(post_modes,'L_tp1')[[1]] %>% apply(c(2,3),quantile,.99) %>% .[,new_order]
+    ideal_pts_low <- ideal_pts_low*sign_flip
+    ideal_pts_high <- ideal_pts_high*sign_flip
+    
+    # now pull the lowest low and highest high
+    ideal_pts_low <- apply(ideal_pts_low,2,function(c) c[which(c==max(c))])
+    ideal_pts_high <- apply(ideal_pts_high,2,function(c) c[which(c==min(c))])
+    ideal_pts_mean <- rstan::extract(post_modes,'L_tp1')[[1]] %>% apply(c(2,3),median) %>% .[,new_order]
+    ideal_pts_mean <- ideal_pts_mean * sign_flip
     time_var_restrict <-  max(apply(this_params[,grepl(pattern = 'time_var_restrict',x=all_params)],2,quantile,.95)[new_order])
-    # constrain any ideal points that are always positive or always negative
-    ideal_pts_low <- rstan::extract(post_modes,'L_tp1')[[1]] %>% apply(3,quantile,.05) %>% .[new_order]
-    ideal_pts_high <- rstan::extract(post_modes,'L_tp1')[[1]] %>% apply(3,quantile,.95) %>% .[new_order]
-    ideal_pts_mean <- rstan::extract(post_modes,'L_tp1')[[1]] %>% apply(3,mean) %>% .[new_order]
-    sign_match <- sign(ideal_pts_low) == sign(ideal_pts_high)
-    constrain_mean <- which(sign_match)
-    # need to select the largest single one if no confidence intervals that don't cross zero
-    if(length(constrain_mean)==0) {
-      constrain_mean <- which(ideal_pts_mean==min(ideal_pts_mean))
-    }
-    if(length(constrain_mean)>1) {
-      constrain_mean <- constrain_mean[abs(ideal_pts_mean[constrain_mean])==max(abs(ideal_pts_mean[constrain_mean]))]
+    
+    if(this_data$time_proc %in% c(2,3)) {
+      # this is just additional for these models
+      col_high <- which(ideal_pts_mean==max(ideal_pts_mean),arr.ind=T)
+      col_low <- which(ideal_pts_mean==min(ideal_pts_mean),arr.ind=T)
+      restrict_mean_ind_high_max <- col_high
+      restrict_mean_ind_high_min <- c(which(ideal_pts_mean[,col_high[,2]]==min(ideal_pts_mean[,col_high[,2]])),
+                                      col_high[,2])
+      restrict_mean_ind_low_min <- col_low
+      restrict_mean_ind_low_max <- c(which(ideal_pts_mean[,col_low[,2]]==max(ideal_pts_mean[,col_low[,2]])),
+                                     col_low[,2])
+      
+    } else if(this_data$time_proc==4) {
+
+      if(fixtype=='vb_full') {
+        col_high <- which(ideal_pts_mean==max(ideal_pts_mean),arr.ind=T)
+        col_low <- which(ideal_pts_mean==min(ideal_pts_mean),arr.ind=T)
+        restrict_mean_ind_high_max <- col_high
+        restrict_mean_ind_high_min <- c(which(ideal_pts_mean[,col_high[,2]]==min(ideal_pts_mean[,col_high[,2]])),
+                                        col_high[,2])
+        restrict_mean_ind_low_min <- col_low
+        restrict_mean_ind_low_max <- c(which(ideal_pts_mean[,col_low[,2]]==max(ideal_pts_mean[,col_low[,2]])),
+                                       col_low[,2])
+      } else {
+        # just constrain the lowest one that we fixed
+        restrict_mean_ind_high_max <- c(which(ideal_pts_mean[,ncol(ideal_pts_mean)]==max(ideal_pts_mean[,ncol(ideal_pts_mean)])),
+                               ncol(ideal_pts_mean))
+        restrict_mean_ind_high_min <- c(which(ideal_pts_mean[,ncol(ideal_pts_mean)]==min(ideal_pts_mean[,ncol(ideal_pts_mean)])),
+                                        ncol(ideal_pts_mean))
+        restrict_mean_ind_low_min <- c(which(ideal_pts_mean[,ncol(ideal_pts_mean)-1]==min(ideal_pts_mean[,ncol(ideal_pts_mean)-1])),
+                                    ncol(ideal_pts_mean)-1)
+        restrict_mean_ind_low_max <- c(which(ideal_pts_mean[,ncol(ideal_pts_mean)-1]==max(ideal_pts_mean[,ncol(ideal_pts_mean)-1])),
+                                       ncol(ideal_pts_mean)-1)
+      }
+      
     }
     
-    restrict_mean <- ideal_pts_mean[constrain_mean]
+    # need four indices for GP fixing
+    restrict_mean_ind <- c(restrict_mean_ind_high_max,
+                           restrict_mean_ind_high_min,
+                           restrict_mean_ind_low_max,
+                           restrict_mean_ind_low_min)
+    restrict_mean_val <- c(ideal_pts_mean[restrict_mean_ind[1],restrict_mean_ind[2]] -
+                             ideal_pts_mean[restrict_mean_ind[5],restrict_mean_ind[6]],
+                           ideal_pts_mean[restrict_mean_ind[3],restrict_mean_ind[4]] -
+                             ideal_pts_mean[restrict_mean_ind[7],restrict_mean_ind[8]])
     
-    object@restrict_mean_val <- restrict_mean
-    object@restrict_mean_ind <- constrain_mean
+    if(fixtype=='vb_partial' && this_data$time_proc==4) {
+      if(restrict_mean_val[1]<0) {
+        restrict_mean_val <- restrict_mean_val * -1
+      }
+    } else if(fixtype=='vb_partial' && this_data$time_proc!=4) {
+      if(sign(person[to_constrain_high])<0) {
+        restrict_mean_val <- restrict_mean_val * -1
+      }
+    }
+    
+    
+
+    object@restrict_mean_val <- restrict_mean_val
+    object@restrict_mean_ind <- restrict_mean_ind
     object@restrict_var_high <- time_var_restrict
   }
 
@@ -217,12 +290,14 @@
     # reorder group parameters
     object@score_matrix <- mutate(ungroup(object@score_matrix), 
                                   group_id=factor(!! quo(group_id)),
-                                  group_id=fct_relevel(!!quo(group_id),to_constrain_low,to_constrain_high,
+                                  group_id=fct_relevel(!!quo(group_id),as.character(to_constrain_low),
+                                                       as.character(to_constrain_high),
                                                        after=length(levels(!!quo(group_id)))))
   } else {
     object@score_matrix <- mutate(ungroup(object@score_matrix), 
                                   person_id=factor(!! quo(person_id)),
-                                  person_id=fct_relevel(!!quo(person_id),to_constrain_low,to_constrain_high,
+                                  person_id=fct_relevel(!!quo(person_id),as.character(to_constrain_low),
+                                                        as.character(to_constrain_high),
                                                        after=length(levels(!!quo(person_id)))))
   }
   
@@ -239,222 +314,6 @@
   
 }
 
-#' #' Function that works with id_model to re-arrange bills or personlators to constrain for identification
-#' .id_params_constrain_guided_inflate <- function(lookat_params=NULL,restrict_params=NULL,nfix=NULL,x=NULL) {
-#' 
-#'   restrict_params <- sort(restrict_params,decreasing=TRUE)
-#'   
-#'   #Create row.names for the vote matrix to preserve order
-#'   
-#'   if(is.null(row.names(x))) {
-#'     row.names(x) <- as.character(1:nrow(x))
-#'   } 
-#'   if(is.null(colnames(x))) {
-#'     colnames(x) <- as.character(1:ncol(x))
-#'   }
-#'   
-#'   # Use different identifications depending on whether we want to do bills, personlators or both
-#'   if(('person' %in% restrict_params) && ('items' %in% restrict_params)) {
-#'     
-#'     # First pull the person fixes
-#'     person_est <- lookat_params[,grepl('L_free\\[',colnames(lookat_params))]
-#'     person_est <- person_est %>% as_data_frame %>% gather(param_name,value) %>% group_by(param_name) %>% 
-#'       summarize(avg=mean(value),high=quantile(value,0.95),low=quantile(value,0.05),sd=sd(value),interval=high-low)
-#'     
-#'     person <- arrange(person_est,desc(avg))
-#'     keep_rows <- as.numeric(stringr::str_extract(person$param_name,'[0-9]+')[1:nfix])
-#'     x <- rbind(x[-keep_rows,],x[keep_rows,])
-#'     
-#'     # Then do the bill fixes
-#'     
-#'     sigmas_est <- lookat_params[,grepl('sigma\\[',colnames(lookat_params))]
-#'     sigmas_est <- sigmas_est %>% as_data_frame %>% gather(param_name,value) %>% group_by(param_name) %>% 
-#'       summarize(avg=mean(value),high=quantile(value,0.95),low=quantile(value,0.05),sd=sd(value),interval=high-low)
-#'     
-#'     sigmas <- arrange(sigmas_est,avg)
-#'     sigmas_est_abs <- lookat_params[,grepl('sigma_abs',colnames(lookat_params))]
-#'     sigmas_est_abs <- sigmas_est_abs %>% as_data_frame %>% gather(param_name,value) %>% group_by(param_name) %>% 
-#'       summarize(avg=mean(value),high=quantile(value,0.95),low=quantile(value,0.05),sd=sd(value),interval=high-low)
-#'     
-#'     sigmas_abs <- arrange(sigmas_est_abs,avg)
-#'     if((mean(sigmas$avg[1:nfix])<mean(sigmas_abs$avg[1:nfix])) & (mean(sigmas$interval[1:nfix])<mean(sigmas_abs$interval[1:nfix]))) {
-#'       keep_cols <- as.numeric(stringr::str_extract(sigmas$param_name,'[0-9]+')[1:nfix])
-#'       param_fix <- c('person','items','sigma')
-#'     } else {
-#'       keep_cols <- as.numeric(stringr::str_extract(sigmas_abs$param_name,'[0-9]+')[1:nfix])
-#'       param_fix <- c('person','items','sigma_abs')
-#'     }
-#'     x <- cbind(x[,-keep_cols],x[,keep_cols])
-#'     
-#'     return(list(restrict=list(restrict_l=nfix,restrict_b=nfix),matrix=x,param_fix=param_fix,
-#'                 restrict_vals=c(sigmas_abs$avg[1:nfix],
-#'                                 sigmas$avg[1:nfix],
-#'                                 person$avg[1:nfix]),
-#'                 unrestricted=lookat_params,
-#'                 restrict_person=keep_rows,
-#'                 restrict_bills=keep_cols))
-#'     
-#'   } else if('person' %in% restrict_params) {
-#'     person_est <- lookat_params[,grepl('L_free\\[',colnames(lookat_params))]
-#'     person_est <- person_est %>% as_data_frame %>% gather(param_name,value) %>% group_by(param_name) %>% 
-#'       summarize(avg=mean(value),high=quantile(value,0.95),low=quantile(value,0.05),sd=sd(value),interval=high-low)
-#'     
-#'     person <- arrange(person_est,desc(avg))
-#'     keep_rows_low <- as.numeric(stringr::str_extract(person$param_name,'[0-9]+')[1:nfix])
-#'     person <- arrange(person_est,avg)
-#'     keep_rows_high <- as.numeric(stringr::str_extract(person$param_name,'[0-9]+')[1:nfix])
-#'     x <- rbind(x[-c(keep_rows_high,keep_rows_low),],x[c(keep_rows_high,keep_rows_low),])
-#'     param_fix <- 'person'
-#'     return(list(restrict=list(restrict=nfix),matrix=x,param_fix=param_fix,
-#'            restrict_vals=person$avg[1:nfix],
-#'            unrestricted=lookat_params,
-#'            restrict_person=c(keep_rows_high,keep_rows_low),
-#'            restrict_bills='None'))
-#'   } else if('items' %in% restrict_params) {
-#'     sigmas_est <- lookat_params[,grepl('sigma\\[',colnames(lookat_params))]
-#'     sigmas_est <- sigmas_est %>% as_data_frame %>% gather(param_name,value) %>% group_by(param_name) %>% 
-#'       summarize(avg=mean(value),high=quantile(value,0.95),low=quantile(value,0.05),sd=sd(value),interval=high-low)
-#'     
-#'     sigmas <- arrange(sigmas_est,avg)
-#'     sigmas_est_abs <- lookat_params[,grepl('sigma_abs',colnames(lookat_params))]
-#'     sigmas_est_abs <- sigmas_est_abs %>% as_data_frame %>% gather(param_name,value) %>% group_by(param_name) %>% 
-#'       summarize(avg=mean(value),high=quantile(value,0.95),low=quantile(value,0.05),sd=sd(value),interval=high-low)
-#'     
-#'     sigmas_abs <- arrange(sigmas_est_abs,avg)
-#'     if((mean(sigmas$avg[1:nfix])<mean(sigmas_abs$avg[1:nfix])) & (mean(sigmas$interval[1:nfix])<mean(sigmas_abs$interval[1:nfix]))) {
-#'       keep_cols <- as.numeric(stringr::str_extract(sigmas$param_name,'[0-9]+')[1:nfix])
-#'       param_fix <- c('items','sigma')
-#'     } else {
-#'       keep_cols <- as.numeric(stringr::str_extract(sigmas_abs$param_name,'[0-9]+')[1:nfix])
-#'       param_fix <- c('items','sigma_abs')
-#'     }
-#'     x <- cbind(x[,-keep_cols],x[,keep_cols])
-#'     return(list(restrict=list(restrict=nfix),matrix=x,param_fix=param_fix,
-#'            restrict_vals=c(sigmas$avg[1:nfix],
-#'                            sigmas_abs$avg[1:nfix]),
-#'            unrestricted=lookat_params,
-#'            restrict_bills=keep_cols,
-#'            restrict_person='None'))
-#'   } else {
-#'     stop('Improper identification parameters passed to the identification function.')
-#'   }
-#'   
-#'   
-#' }
-#' 
-#' .id_params_constrain_guided_2pl <- function(lookat_params=NULL,restrict_params=NULL,nfix=NULL,x=NULL) {
-#'   
-#'   restrict_params <- sort(restrict_params,decreasing=TRUE)
-#'   
-#'   #Create row.names for the vote matrix to preserve order
-#'   
-#'   if(is.null(row.names(x))) {
-#'     row.names(x) <- as.character(1:nrow(x))
-#'   } 
-#'   if(is.null(colnames(x))) {
-#'     colnames(x) <- as.character(1:ncol(x))
-#'   }
-#'   
-#'   # Use different identifications depending on whether we want to do bills, personlators or both
-#'   if(('person' %in% restrict_params) && ('items' %in% restrict_params)) {
-#'     
-#'     # First pull the person fixes
-#'     person_est <- lookat_params[,grepl('L_free\\[',colnames(lookat_params))]
-#'     person_est <- person_est %>% as_data_frame %>% gather(param_name,value) %>% group_by(param_name) %>% 
-#'       summarize(avg=mean(value),high=quantile(value,0.95),low=quantile(value,0.05),sd=sd(value),interval=high-low)
-#'     
-#'     person <- arrange(person_est,desc(avg))
-#'     keep_rows <- as.numeric(stringr::str_extract(person$param_name,'[0-9]+')[1:nfix])
-#'     x <- rbind(x[-keep_rows,],x[keep_rows,])
-#'     
-#'     # Then do the bill fixes
-#'     
-#'     sigmas_est <- lookat_params[,grepl('sigma\\[',colnames(lookat_params))]
-#'     sigmas <- sigmas_est %>% as_data_frame %>% gather(param_name,value) %>% group_by(param_name) %>% 
-#'       summarize(avg=mean(value),high=quantile(value,0.95),low=quantile(value,0.05),sd=sd(value),interval=high-low)
-#' 
-#'     keep_cols <- as.numeric(stringr::str_extract(sigmas$param_name,'[0-9]+')[1:nfix])
-#'     param_fix <- c('person','items','sigma')
-#' 
-#'     x <- cbind(x[,-keep_cols],x[,keep_cols])
-#'     
-#'     return(list(restrict=list(restrict_l=nfix,restrict_b=nfix),matrix=x,param_fix=param_fix,
-#'                 restrict_vals=c(sigmas$avg[1:nfix],
-#'                                 person$avg[1:nfix]),
-#'                 unrestricted=lookat_params,
-#'                 restrict_person=keep_rows,
-#'                 restrict_bills=keep_cols))
-#'     
-#'   } else if('person' %in% restrict_params) {
-#'     person_est <- lookat_params[,grepl('L_free\\[',colnames(lookat_params))]
-#'     person_est <- person_est %>% as_data_frame %>% gather(param_name,value) %>% group_by(param_name) %>% 
-#'       summarize(avg=mean(value),high=quantile(value,0.95),low=quantile(value,0.05),sd=sd(value),interval=high-low)
-#'     
-#'     person <- arrange(person_est,desc(avg))
-#'     keep_rows_low <- as.numeric(stringr::str_extract(person$param_name,'[0-9]+')[1:nfix])
-#'     person <- arrange(person_est,avg)
-#'     keep_rows_high <- as.numeric(stringr::str_extract(person$param_name,'[0-9]+')[1:nfix])
-#'     x <- rbind(x[-c(keep_rows_high,keep_rows_low),],x[c(keep_rows_high,keep_rows_low),])
-#'     param_fix <- 'person'
-#'     return(list(restrict=list(restrict=nfix),matrix=x,param_fix=param_fix,
-#'                 restrict_vals=person$avg[1:nfix],
-#'                 unrestricted=lookat_params,
-#'                 restrict_person=c(keep_rows_high,keep_rows_low),restrict_bills='None'))
-#'   } else if('items' %in% restrict_params) {
-#'     sigmas_est <- lookat_params[,grepl('sigma\\[',colnames(lookat_params))]
-#'     sigmas <- sigmas_est %>% as_data_frame %>% gather(param_name,value) %>% group_by(param_name) %>% 
-#'       summarize(avg=mean(value),high=quantile(value,0.95),low=quantile(value,0.05),sd=sd(value),interval=high-low)
-#' 
-#'      keep_cols <- as.numeric(stringr::str_extract(sigmas$param_name,'[0-9]+')[1:nfix])
-#'       param_fix <- c('items','sigma')
-#' 
-#'     x <- cbind(x[,-keep_cols],x[,keep_cols])
-#'     return(list(restrict=list(restrict=nfix),matrix=x,param_fix=param_fix,
-#'                 restrict_vals=sigmas$avg[1:nfix],
-#'                 unrestricted=lookat_params,
-#'                 restrict_bills=keep_cols,
-#'                 restrict_person='None'))
-#'   } else {
-#'     stop('Improper identification parameters passed to the identification function.')
-#'   }
-#'   
-#'   
-#' }
-#' 
-#' .id_params_pin_unguided_inflate <- function(restrict_params=NULL,nfix=NULL,x=NULL,
-#'                                    restrict_names=NULL,bill_names=NULL,person_names=NULL) {
-#'   
-#'   restrict_params <- sort(restrict_params,decreasing=TRUE)
-#'   
-#'   #Create row.names for the vote matrix to preserve order
-#'   
-#'   if(is.null(row.names(x))) {
-#'     row.names(x) <- as.character(1:nrow(x))
-#'   } 
-#'   if(is.null(colnames(x))) {
-#'     colnames(x) <- as.character(1:ncol(x))
-#'   }
-#'   
-#'   if(('person' %in% restrict_params) && ('items' %in% restrict_params)) {
-#'     if(length(restrict_names)<2) {
-#'       stop('You must pass both a bill and a person name to pin a parameter for both bills and persons.')
-#'     }
-#'     restrict_pos[1] <- which(restrict_names[1]==person_names)
-#'     restrict_pos[2] <- which(restrict_names[2]==bill_names)
-#'     x <- rbind(x[-restrict_pos[1],],x[restrict_pos[1],])
-#'     x <- cbind(x[,-restrict_pos[2]],x[,restrict_pos[2]])
-#'   } else if('person' %in% restrict_params) {
-#'     restrict_pos[1] <- which(restrict_names[1]==person_names)
-#'     x <- rbind(x[-restrict_pos[1],],x[restrict_pos[1],])
-#'   } else if('items' %in% restrict_params) {
-#'     restrict_pos[1] <- which(restrict_names[1]==bill_names)
-#'     x <- cbind(x[,-restrict_pos[1]],x[,restrict_pos[1]])
-#'   }
-#'   
-#'   return(list(restrict=list(matrix=x,param_fix=restrict_params,restrict_vals=restrict_pos)))
-#'   
-#' }
-
 #' Function that figures out what kind of hierarchical model (if any) is being run
 #' @noRd
 .get_hier_type <- function(obj) {
@@ -678,13 +537,18 @@
 }
 
 #' Simple function to provide initial values to Stan given current values of restrict_sd
+#' @importFrom stats optimize
 #' @noRd
 .init_stan <- function(chain_id=NULL,
                        restrict_sd=NULL,
                         person_sd=NULL,
                        num_legis=NULL,
                        diff_high=NULL,
+                       m_sd_par=NULL,
+                       num_diff=NULL,
+                       time_range=NULL,
                        T=NULL,
+                       time_proc=NULL,
                        restrict_var_high=NULL,
                        time_sd=NULL,
                        restrict_var=NULL,
@@ -697,16 +561,39 @@
   } else {
     L_free <- array(person_start[1:(num_legis-2)])
   }
+  
+  # given upper bound on m_sd figure out mean to match real value on the real numbers
+  
+  rev_trans <- function(x,m_sd_par) {
+    m_sd_par[1]*plogis(x) - 1/m_sd_par[2]
+  }
 
   
   if(actual==TRUE) {
     # full run
     if(T>1) {
+      
+      # figure out optimized gp par
+      
+      m_sd_optim <- optimize(f=rev_trans,
+                             interval=c(0,m_sd_par[1]),
+                          m_sd_par=m_sd_par)$objective
+      
       if(restrict_var) {
-        return(list(restrict_high = array(rnorm(n=1,mean=diff_high,sd=restrict_sd)),
-                    L_free = L_free,
-                    L_AR1 = array(runif(n = num_legis,min = -.5,max=.5)),
-                    time_var_restrict = rep(restrict_var_high/2,num_legis)))
+        if(time_proc==4) {
+          return(list(restrict_high = array(rnorm(n=1,mean=diff_high,sd=restrict_sd)),
+                      L_free = L_free,
+                      m_sd=rep(m_sd_optim,num_legis),
+                      time_var=log(rep(num_diff[1]*time_range,num_legis)),
+                      L_AR1 = array(runif(n = num_legis,min = -.5,max=.5)),
+                      time_var_restrict = rep(restrict_var_high/2,num_legis)))
+        } else {
+          return(list(restrict_high = array(rnorm(n=1,mean=diff_high,sd=restrict_sd)),
+                      L_free = L_free,
+                      L_AR1 = array(runif(n = num_legis,min = -.5,max=.5)),
+                      time_var_restrict = rep(restrict_var_high/2,num_legis)))
+        }
+        
       } else {
         return(list(restrict_high = array(rnorm(n=1,mean=diff_high,sd=restrict_sd)),
                     L_free = L_free,
@@ -1654,4 +1541,26 @@ return(as.vector(idx))
     x <- na_if(x,to_na)
   }
   return(x)
-}
\ No newline at end of file
+}
+
+#' Calculate priors for Gaussian processes
+#' @noRd
+.gp_prior <- function(time_points=NULL) {
+
+  # need to calculate minimum and maximum difference between *any* two points
+  diff_time <- diff(time_points)
+  min_diff <- min(diff_time)+1
+  # divide max_diff by 2 to constrain the prior away from very large values
+  max_diff <- abs(time_points[1]-time_points[2])*2
+  
+  # now run the stan program with the data
+  
+  fit <- sampling(object = stanmodels[['gp_prior_tune']], iter=1, warmup=0, chains=1,
+              seed=5838298, algorithm="Fixed_param")
+  params <- extract(fit)
+  
+  return(list(a=c(params$a),
+              b=c(params$b)))
+  
+}
+
