@@ -12,7 +12,14 @@
 #' which side will be voting which way. For that reason, the legislators/persons are colored by their votes/scores to
 #' make it clear.
 #' 
-#' @param object A fitted \code{idealstan} object
+#' To compare across multiple \code{idealstan} models, pass a named list 
+#' \code{list(model1=model1,model2=model2,etc)} to the \code{object} option. 
+#' Note that these comparisons will done by individual persons/groups, so if there are a lot of 
+#' persons/groups, consider using the \code{include} option to only compare a specific set
+#' of persons/groups.
+#' 
+#' @param object A fitted \code{idealstan} object or a named list
+#' of \code{idealstan} objects to compare across models
 #' @param return_data If true, the calculated legislator/bill data is returned along with the plot in a list
 #' @param include Specify a list of person/legislator IDs to include in the plot (all others excluded)
 #' @param high_limit The quantile (number between 0 and 1) for the high end of posterior uncertainty to show in plot
@@ -84,19 +91,45 @@ id_plot_legis <- function(object,return_data=FALSE,
                        show_true=FALSE,group_color=TRUE,
                        hpd_limit=10,
                        sample_persons=NULL,...) {
+  
+  if(class(object)=='idealstan') {
+    person_params <- .prepare_legis_data(object,
+                                         high_limit=high_limit,
+                                         low_limit=low_limit)
+    model_wrap <- FALSE
+    use_groups <- object@use_groups
+  } else {
+    # loop over lists
+    person_params <- lapply(object,.prepare_legis_data,
+                            high_limit=high_limit,
+                            low_limit=low_limit) %>% 
+      bind_rows(.id='Model')
+    
+    check_groups <- sapply(object,function(x) x@use_groups)
+    if(all(check_groups)) {
+      use_groups <- T
+    } else {
+      use_groups <- F
+    }
+    # now we can facet by persons/groups
+    model_wrap <- TRUE
+    group_color <- FALSE
+    # assume all data is the same
+    object <- object[[1]]
+  }
 
-  person_params <- .prepare_legis_data(object,
-                                       high_limit=high_limit,
-                                       low_limit=low_limit)
   
   if(!is.null(include)) {
     person_params <- filter(person_params, person_id %in% include)
   }
   
-  if(group_color) {
+  if(group_color && !model_wrap) {
     groupc <- ~group_id
-  } else {
+  } else if(!group_color && !model_wrap) {
     groupc <- NA
+  } else {
+    # if multiple models, subset by models
+    groupc <- ~Model
   }
 
   # sample for plot only
@@ -257,8 +290,15 @@ id_plot_legis <- function(object,return_data=FALSE,
   
   # Add theme elements
   
-  outplot <- outplot  + theme_minimal() + ylab("") + xlab("") +
-    theme(axis.text.y=element_blank(),panel.grid.major.y = element_blank()) + coord_flip() 
+  outplot <- outplot  +  ylab("") + xlab("") +
+    theme(axis.text.y=element_blank(),panel.grid.major.y = element_blank(),
+          strip.background = element_blank(),
+          panel.background = element_blank()) + coord_flip() 
+  
+  if(model_wrap) {
+    # facet by groups/persons
+    outplot <- outplot + facet_wrap(~group_id)
+  }
 
   
   if(return_data==TRUE) {
@@ -367,7 +407,10 @@ id_plot_legis_var <- function(object,return_data=FALSE,
                     color=group_id),size=text_size_group,show.legend = FALSE) 
   } else if(group_color==FALSE) {
     outplot <- person_params %>% ggplot() +
-      geom_linerange(aes(x=reorder(person_id,median_pt),ymin=low_pt,ymax=high_pt),alpha=person_ci_alpha) +
+      geom_linerange(aes(x=reorder(person_id,
+                                   median_pt),
+                         ymin=low_pt,ymax=high_pt),
+                     alpha=person_ci_alpha) +
       geom_text(aes(x=reorder(person_id,median_pt),y=median_pt,
                           label=reorder(group_id,median_pt)),size=text_size_group)
     
@@ -426,7 +469,9 @@ id_plot_legis_var <- function(object,return_data=FALSE,
 #' which side will be voting which way. For that reason, the legislators/persons are colored by their votes/scores to
 #' make it clear.
 #' 
-#' @param object A fitted \code{idealstan} object
+#' @param object A fitted \code{idealstan} object or a named list of \code{idealstan}
+#' objects if the plot is supposed to show a comparison of different fitted \code{idealstan}
+#' models (see Time Series vignette)
 #' @param return_data If true, the calculated legislator/bill data is returned along with the plot in a list
 #' @param include Specify a list of person/legislator IDs to include in the plot (all others excluded)
 #' @param item_plot The value of the item/bill for which to plot its midpoint (character value)
@@ -506,9 +551,32 @@ id_plot_legis_dyn <- function(object,return_data=FALSE,
   
   person_labels <- quo(person_id)
   group_labels <- quo(group_id)
-  person_params <- .prepare_legis_data(object,
-                                       high_limit=high_limit,
-                                       low_limit=low_limit) 
+  already_facet <- FALSE
+  
+  if(class(object)=='idealstan') {
+    person_params <- .prepare_legis_data(object,
+                                         high_limit=high_limit,
+                                         low_limit=low_limit)
+    model_wrap <- FALSE
+    use_groups <- object@use_groups
+  } else {
+    # loop over lists
+    person_params <- lapply(object,.prepare_legis_data,
+                            high_limit=high_limit,
+                            low_limit=low_limit) %>% 
+      bind_rows(.id='Model')
+    
+    check_groups <- sapply(object,function(x) x@use_groups)
+    if(all(check_groups)) {
+      use_groups <- T
+    } else {
+      use_groups <- F
+    }
+    # now we can facet by persons/groups
+    model_wrap <- TRUE
+    # assume all data is the same
+    object <- object[[1]]
+  }
   
   # create item plot data
   
@@ -565,7 +633,7 @@ id_plot_legis_dyn <- function(object,return_data=FALSE,
   }
   
   if(!is.null(include)) {
-    if(object@use_groups) {
+    if(use_groups) {
       person_params <- filter(person_params, group_id %in% include)
     } else {
       person_params <- filter(person_params, person_id %in% include)
@@ -573,10 +641,18 @@ id_plot_legis_dyn <- function(object,return_data=FALSE,
     
   }
   
-  if(object@use_groups) {
+  if(use_groups && !model_wrap) {
     base_id <- ~group_id
-  } else {
+  } else if(!use_groups && !model_wrap) {
     base_id <- ~person_id
+  } else {
+    # set base id to the model type
+    base_id <- ~Model
+    if(use_groups) {
+      wrap_id <- ~group_id
+    } else {
+      wrap_id <- ~person_id
+    }
   }
   
   # allow the option of plotting "true" ideal points instead of estimated ones as lines
@@ -589,8 +665,9 @@ id_plot_legis_dyn <- function(object,return_data=FALSE,
       # need to flip for identification
       mutate(time_id=as.numeric(time_id),
              person_id=factor(person_id),
-             person_id=fct_relevel(person_id,object@score_data@restrict_ind_low,
-                                   object@score_data@restrict_ind_high,
+             person_id=fct_relevel(person_id,
+                                   as.character(object@score_data@restrict_ind_low),
+                                   as.character(object@score_data@restrict_ind_high),
                                    
                                    after=length(levels(person_id))))
     person_params <- left_join(person_params,true_pts,by=c("person_id","time_id"))
@@ -621,12 +698,20 @@ id_plot_legis_dyn <- function(object,return_data=FALSE,
     
   } else {
     if(group_color) {
+      if(model_wrap) {
+        outplot <- outplot + 
+          geom_line(aes_(y=~median_pt,group=base_id,
+                         colour=~Model),
+                    alpha=person_ci_alpha,
+                    size=line_size)
+      } else {
+        outplot <- outplot + 
+          geom_line(aes_(y=~median_pt,group=base_id,
+                         colour=~group_id),
+                    alpha=person_ci_alpha,
+                    size=line_size)
+      }
       
-      outplot <- outplot + 
-        geom_line(aes_(y=~median_pt,group=base_id,
-                       colour=~group_id),
-                  alpha=person_ci_alpha,
-                  size=line_size)
     } else {
       
       outplot <- outplot + 
@@ -648,13 +733,26 @@ id_plot_legis_dyn <- function(object,return_data=FALSE,
       theme(legend.position = 'bottom')
     
     #Whether or not to add a facet_wrap
-    
-    if(item_plot_type=='both' & length(item_plot)>1) {
-      outplot <- outplot + facet_wrap(~item_name + item_type,dir='v')
+    # Note interferes with model_wrap
+    if(item_plot_type=='both' && length(item_plot)>1) {
+      if(model_wrap) {
+        outplot <- outplot + facet_wrap(update(wrap_id,~. + item_name + item_type),dir='v',
+                                        ncol=length(pull(person_params,Model)))
+      } else {
+        outplot <- outplot + facet_wrap(~item_name + item_type ,dir='v')
+      }
+      
     } else if(item_plot_type=='both') {
-      outplot <- outplot + facet_wrap(~item_type,dir='v') 
+      if(model_wrap) {
+        outplot <- outplot + facet_wrap(update(wrap_id,~ . + item_type),dir='v',
+                                        ncol=length(pull(person_params,Model)))
+      } else {
+        outplot <- outplot + facet_wrap(~item_type,dir='v')
+      }
+       
     } 
-    
+    # record state
+    already_facet <- TRUE
     
   }
 
@@ -664,8 +762,12 @@ id_plot_legis_dyn <- function(object,return_data=FALSE,
   if(plot_text==TRUE) {
     
     # need new data that scatters names around the plot
+    if(model_wrap) {
+      sampled_data <- group_by(person_params,!!as_quosure(base_id),!!as_quosure(wrap_id)) %>% sample_n(1)
+    } else {
+      sampled_data <- group_by(person_params,!!as_quosure(base_id)) %>% sample_n(1)
+    }
     
-    sampled_data <- group_by(person_params,!!as_quosure(base_id)) %>% sample_n(1)
     
     if(!is.null(highlight)) {
       
@@ -699,6 +801,13 @@ id_plot_legis_dyn <- function(object,return_data=FALSE,
     theme_minimal() + ylab("Ideal Point Scale") + xlab("") +
     theme(panel.grid= element_blank())
   
+  # if not already facetted, facet
+  
+  if(!already_facet && model_wrap) {
+    outplot <- outplot + facet_wrap(wrap_id,scales = 'free_y',
+                                    ncol=length(pull(person_params,Model)))
+  }
+  
   return(outplot)
   
 }
@@ -894,8 +1003,7 @@ id_plot_rhats <- function(obj) {
 #' column names otherwise. 
 #' 
 #' @param object A fitted \code{idealstan} object
-#' @param cov_type Either \code{'person_cov'} for person-level hierarchical parameters,
-#' \code{'group_cov'} for group-level hierarchical parameters,
+#' @param cov_type Either \code{'person_cov'} for person or group-level hierarchical parameters,
 #' \code{'discrim_reg_cov'} for bill/item discrimination parameters from regular (non-inflated) model, and 
 #' \code{'discrim_infl_cov'} for bill/item discrimination parameters from inflated model.
 #' @param filter_cov A character vector of coefficients from covariate plots to exclude from
